from azmessaging import default_settings as settings
from .bases import Reader
from .sms import SMSReaderMixin, SMSConfig
from .telegram import TelegramReaderMixin, TelegramConfig
from .pushnotifications import PushNotificationReaderMixin, PushNotificationConfig


class DefaultReader(Reader, SMSReaderMixin, TelegramReaderMixin, PushNotificationReaderMixin):

    def get_push_notification_config_class(self) -> type(PushNotificationConfig):
        return PushNotificationConfig

    def get_push_notification_config(self, identifier) -> PushNotificationConfig:
        return self.get_push_notification_config_class()(
            default=settings.PUSH_NOTIFICATION_CONFIG['DEFAULT_SERVICE_PROVIDER'],
            priorities=settings.PUSH_NOTIFICATION_CONFIG['PRIORITY_SERVICE_PROVIDER'],
            service_providers=settings.PUSH_NOTIFICATION_CONFIG['SERVICE_PROVIDER'],
        )

    def get_telegram_config(self, identifier) -> TelegramConfig:
        return self.get_telegram_config_class()(
            default=settings.TELEGRAM_CONFIG['DEFAULT_SERVICE_PROVIDER'],
            priorities=settings.TELEGRAM_CONFIG['PRIORITY_SERVICE_PROVIDER'],
            service_providers=settings.TELEGRAM_CONFIG['SERVICE_PROVIDER'],
        )

    def get_telegram_config_class(self) -> type(TelegramConfig):
        return TelegramConfig

    def get_sms_config_class(self) -> type(SMSConfig):
        return SMSConfig

    def get_sms_config(self, identifier) -> SMSConfig:
        return self.get_sms_config_class()(
            default=settings.SMS_CONFIG['DEFAULT_SERVICE_PROVIDER'],
            priorities=settings.SMS_CONFIG['PRIORITY_SERVICE_PROVIDER'],
            service_providers=settings.SMS_CONFIG['SERVICE_PROVIDER'],
            white_list_countries=settings.SMS_CONFIG['WHITE_LIST'],
            black_list_countries=settings.SMS_CONFIG['BLACK_LIST'],
        )



import fcm_django

if fcm_django.__version__ >= '1.0.0':
    raise ImportError('FCMDjango version must be lower than version 1.')

from fcm_django.models import FCMDevice

from .pushabstract import PushNotificationAPI


class FCMDjangoAPI(PushNotificationAPI):

    def __init__(self, api_key, **kwargs):
        super(FCMDjangoAPI, self).__init__(**kwargs)
        self.api_key = api_key

    def bulk_send(self, title: str, message: str, image_url: str, receivers: list,
                  payload_data: dict):
        try:
            fcm_devices = FCMDevice.objects.filter(
                registration_id__in=receivers,
            ).distinct()
            sent_result = fcm_devices.send_message(
                title=title,
                body=message,
                icon=image_url,
                data=payload_data,
                api_key=self.api_key,
            )
            # TODO: handle response to log and etc...
        except Exception as inst:
            # obj.description_for_admin = 'type: {0}\nargs: {1}\ninstance: {2}'.format(type(inst), inst.args, inst)
            # obj.save()
            # TODO: handle response to log and etc...
            pass

    def send(self, title: str, message: str, image_url: str, receiver: str, payload_data: dict):
        self.bulk_send(title, message, image_url, [receiver], payload_data)



import abc
import six


@six.add_metaclass(abc.ABCMeta)
class PushNotificationAPI:

    def __init__(self, **kwargs):
        pass

    @abc.abstractmethod
    def send(self, title: str, message: str, image_url: str, receiver: str, payload_data: dict):
        """

        :param title:
        :param message:
        :param image_url:
        :param receiver:
        :param payload_data: use for data in push notification
        :return:
        """
        pass

    @abc.abstractmethod
    def bulk_send(self, title: str, message: str, image_url: str, receivers: list, payload_data: dict):
        """

        :param title:
        :param message:
        :param image_url:
        :param receivers:
        :param payload_data: use for data in push notification
        :return:
        """
        pass



from .base import BaseNotificationChannel


class PushNotificationChannel(BaseNotificationChannel):
    """
    structure: []
    for example:
    ["token_1", "token_2", "token_3", ]
    """
    _receivers = []

    def __init__(self, title, image_url, payload_data, **kwargs):
        super(PushNotificationChannel, self).__init__(**kwargs)
        self.title = title
        self.image_url = image_url
        self.payload_data = payload_data

    def _send_msg(self, title, message, image_url, receivers, payload_data):
        sender = self.reader.get_push_notification_sender(self.identifier)
        sender.bulk_send(
            title=title,
            message=message,
            image_url=image_url,
            receivers=receivers,
            payload_data=payload_data,
        )

    def get_message(self):
        return self.raw_message

    def set_receivers(self, receivers: list):
        self._receivers = receivers

    def get_receivers(self):
        return self._receivers

    def notify(self):
        """Sends the notification."""
        self._send_msg(
            title=self.title,
            message=self.get_message(),
            image_url=self.image_url,
            receivers=self.get_receivers(),
            payload_data=self.payload_data
        )



import abc

import six

from azmessaging import default_settings as settings
from azmessaging.utils import import_class


@six.add_metaclass(abc.ABCMeta)
class Reader:

    def klass(self, channel: str, identifier: str) -> dict:
        return import_class(settings.CHANNEL_CLASS[channel.upper()])
